package edu.columbia.cs.irt.rfidentify.display;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.MouseInfo;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author "Blake Arnold"
 *This class sets up the display for presenting the name of the speaker
 *
 */
public class DisplayName implements Observer {

	private final Logger logger = LoggerFactory.getLogger(DisplayName.class);

	private JLabel name;
	private String displayName;
	private boolean fullScreen;
	private JFrame undecJ;
	private JFrame decJ;
	private JPanel myPanel;
	/**
	 * Creates class
	 * @param displayName name of window. Usually not displayed if in full screen
	 */
	public DisplayName(String displayName) {
		this.displayName = displayName;
		this.name = new JLabel("Press spacebar to enter fullscreen", JLabel.CENTER);
	}

	/**
	 * Initial function to set up display. Determines which display to go full screen on
	 */
	public void setupDisplay() {

		// Determine if full-screen mode is supported directly
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();

		GraphicsDevice[] gs = ge.getScreenDevices();
		logger.info("screens: " + gs.length);
		GraphicsDevice gd = MouseInfo.getPointerInfo().getDevice();
		
		//We set up two Jframes, one for full screen
		// and one for windowed.
		undecJ = new JFrame(gd.getDefaultConfiguration());
		undecJ.setExtendedState(JFrame.MAXIMIZED_BOTH);
		undecJ.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		undecJ.setUndecorated(true);
		
		
		decJ = new JFrame(gd.getDefaultConfiguration());
		decJ.setExtendedState(JFrame.MAXIMIZED_BOTH);
		decJ.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		myPanel = new JPanel();
		setupFrame(myPanel);
		
		decJ.add(myPanel);
		
		
		decJ.pack();
		undecJ.pack();

		decJ.setVisible(true);
		
		fullScreen = false;
		
	}

	/**
	 * Loads components of display
	 * @param f panel to put components on
	 */
	public void setupFrame(final JPanel f) {

		f.setLayout(new BorderLayout());
		decJ.setName(displayName);
		// Create a hidden button in upper left corner
		// that leaves full-screen mode
		JButton btn = new JButton();
		

		btn.setSize(100, 50);
		btn.setBackground(Color.BLACK);
		btn.setBorderPainted(false);
		
		ActionListener fullScreenListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				// Return to normal windowed mode
				GraphicsDevice gd = MouseInfo.getPointerInfo().getDevice();
				if(fullScreen){
					decJ.add(myPanel);
					undecJ.setVisible(false);
					decJ.setVisible(true);
					decJ.add(myPanel);
					gd.setFullScreenWindow(null);
					fullScreen=false;
				} else {
					undecJ.add(myPanel);
					undecJ.setVisible(true);
					decJ.setVisible(false);
					gd.setFullScreenWindow(undecJ);
					
					fullScreen = true;
				}
				
				

			}

		};
		btn.addActionListener(fullScreenListener);
		// This allows spacebar to get in and out of full screen
		f.addKeyListener( new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {
				logger.info(e.getKeyCode() + " ");
				if(e.getKeyCode()==0) {
					// Return to normal windowed mode
					GraphicsDevice gd = MouseInfo.getPointerInfo().getDevice();
					if(fullScreen){
						decJ.add(myPanel);
						undecJ.setVisible(false);
						decJ.setVisible(true);
						decJ.add(myPanel);
						gd.setFullScreenWindow(null);
						fullScreen=false;
					} else {
						undecJ.add(myPanel);
						undecJ.setVisible(true);
						decJ.setVisible(false);
						gd.setFullScreenWindow(undecJ);
						
						fullScreen = true;
					}
				}
				
			}

			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void keyTyped(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
			
		});

		f.add(btn, BorderLayout.CENTER);
		

		Font font = new Font("Arial Bold", Font.ITALIC, 72);
		name.setFont(font);
		name.setForeground(Color.white);
		f.setBackground(Color.black);
		f.add(name, BorderLayout.CENTER);

		fullScreen = true;
		

	}

	/**
	 * Function updates
	 */
	@Override
	public void update(Observable o, Object req) {

		name.setText((String) req);
		myPanel.repaint();

	}
}
