package edu.columbia.cs.irt.rfidentify.display;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Observable;
import java.util.Scanner;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.MethodNotSupportedException;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentProducer;
import org.apache.http.entity.EntityTemplate;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpRequestHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.columbia.cs.irt.rfidentify.dbhandler.DBAccess;

/**
 * Used to update handlers that a tag has been swiped
 * 
 * @author Blake Arnold
 * 
 */
public class IdTagHandler extends Observable implements HttpRequestHandler {

	private DBAccess dbAccess;
	private Logger logger = LoggerFactory.getLogger(IdTagHandler.class);

	/**
	 * Initializes new handler
	 * 
	 * @param dbAccess
	 *            Access to local database
	 */
	public IdTagHandler(DBAccess dbAccess) {
		this.dbAccess = dbAccess;
	}

	@Override
	public void handle(org.apache.http.HttpRequest request,
			HttpResponse response, HttpContext context) throws HttpException,
			IOException {
		String method = request.getRequestLine().getMethod().toUpperCase(
				Locale.ENGLISH);
		if (!method.equals("GET") && !method.equals("HEAD")
				&& !method.equals("POST")) {
			throw new MethodNotSupportedException(method
					+ " method not supported");
		}
		String target = request.getRequestLine().getUri();

		logger.info(target);

		EntityTemplate body = null;
		String parameters = "";
		// Strip away the query part part if found
		int index = target.indexOf("?");
		if (index != -1) {
			parameters = target.substring(index + 1);
		}

		List<NameValuePair> params = new ArrayList<NameValuePair>();
		HashMap<String, String> paramsHash = new HashMap<String, String>();
		URLEncodedUtils.parse(params, new Scanner(parameters), "UTF-8");
		for (NameValuePair param : params) {
			paramsHash.put(param.getName(), param.getValue());
		}

		final String idTagNum = paramsHash.get("idTagNum");
		final String idName = dbAccess
				.getName(idTagNum == null ? "" : idTagNum);

		if (idName != null) {
			logger.info("name found for idtag: " + idTagNum + " " + idName);
			body = new EntityTemplate(new ContentProducer() {

				public void writeTo(final OutputStream outstream)
						throws IOException {
					OutputStreamWriter writer = new OutputStreamWriter(
							outstream, "UTF-8");
					writer.write("<html><body><h1>");
					writer.write(idName);
					writer.write("</h1></body></html>");
					writer.flush();
				}

			});
			this.setChanged();
			this.notifyObservers(idName);
		} else {
			logger.info("no name found for idtag: " + idTagNum);
			body = new EntityTemplate(new ContentProducer() {

				public void writeTo(final OutputStream outstream)
						throws IOException {
					OutputStreamWriter writer = new OutputStreamWriter(
							outstream, "UTF-8");
					writer.write("<html><body><h1>");
					writer.write("No Name found for id: " + idTagNum);
					writer.write("</h1></body></html>");
					writer.flush();
				}

			});

		}

		response.setStatusCode(HttpStatus.SC_OK);

		body.setContentType("text/html; charset=UTF-8");
		response.setEntity(body);
	}

}
