package edu.columbia.cs.irt.rfidentify.display;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.textmarks.api2client.TextMarksV2APIClient;

import edu.columbia.cs.irt.rfidentify.core.PropertyLoader;


/**
 * Interface for textMarks
 * @author Blake Arnold
 *
 */
public class TextMarksHandler implements Observer, Runnable{
	private String apiKey;
	private String user;
	private String pass;
	private String keyWord;
	private String request;
	Logger logger = LoggerFactory.getLogger(TextMarksHandler.class);
	
	public TextMarksHandler(){
		this.loadProperties();
	}
	
	/**
	 * New thread is spawned to send as it is blocking.
	 */
	@Override
	public void run() {
		this.sendText("New Speaker Name: " + request);
		
	}
	
	public void sendText(String text)	{
		try
		{
			TextMarksV2APIClient tmapi;
			Map<String,Object> msoParams;
			// Broadcast a message to a TextMark group:
			logger.info("Broadcasting a message to a TextMark group..." + text);
			tmapi = new TextMarksV2APIClient();
			msoParams = new HashMap<String, Object>(); 
			tmapi.setApiKey(apiKey);
			tmapi.setAuthUser(user); // (or my TextMarks phone#)
			tmapi.setAuthPass(pass);
			msoParams.put("tm", keyWord);
			msoParams.put("msg", text);
		}
		catch (Exception ex) {
			logger.error("Exception cought:\n"+ ex.toString());
		}
	}
	@Override
	public void update(Observable o, Object req) {
		request = (String) req;
		Thread thread = new Thread(this);
        thread.start();
			
		
	}
	
	private void loadProperties(){
		String pName = "textmarks-api.properties";
		Properties props;
		try{
			props = PropertyLoader.loadProps(pName);
			apiKey = props.getProperty("ApiKey");
			user = props.getProperty("AuthUser");
			pass = props.getProperty("AuthPass");
			keyWord = props.getProperty("keyword");
			} catch (FileNotFoundException e) {
				logger.error("File " + pName + " does not exist" + e.getMessage(), e);
			} catch (IOException e) {
				logger.error("Error while loading property file " + pName
						+ " Loading defaults" + e.getMessage(), e);
			}

	}

}
