package edu.columbia.cs.irt.rfidentify.registration;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Scanner;

import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.MethodNotSupportedException;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentProducer;
import org.apache.http.entity.EntityTemplate;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpRequestHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.columbia.cs.irt.rfidentify.dbhandler.DBAccess;
import edu.columbia.cs.irt.rfidentify.server.HttpConstants;


/**
 * Handles registration at kiosks
 * 
 * @author "Blake Arnold"
 * 
 */
public class RegistrationHandler implements HttpConstants,
		HttpRequestHandler {
	private Logger logger = LoggerFactory.getLogger(RegistrationHandler.class);
	private DBAccess dbAccess;
	private String lastSwipe;

	/**
	 * initializes handler
	 * 
	 * @param dbAccess
	 *            local database
	 */
	public RegistrationHandler(DBAccess dbAccess) {
		this.dbAccess = dbAccess;
	}


	@Override
	public void handle(org.apache.http.HttpRequest request,
			HttpResponse response, HttpContext context) throws HttpException,
			IOException {

		String method = request.getRequestLine().getMethod().toUpperCase(
				Locale.ENGLISH);
		String target = request.getRequestLine().getUri();
		if (!method.equals("GET") && !method.equals("HEAD")
				&& !method.equals("POST")) {
			throw new MethodNotSupportedException(method
					+ " method not supported");
		}

		if (method.equals("GET")) {
			String parameters = "";
	        //Strip away the query part part if found
	        int index = target.indexOf("?");
	        if (index != -1) {
	            parameters = target.substring(index+1);
	        }

			List<NameValuePair> params = new ArrayList<NameValuePair>();
			HashMap<String, String> paramsHash = new HashMap<String, String>();
			URLEncodedUtils.parse(params, new Scanner(parameters), "UTF-8");
			for (NameValuePair param : params) {
				paramsHash.put(param.getName(), param.getValue());
			}

			String idTagNum = paramsHash.get("idTagNum");

			// Receiving new idTag
			if (idTagNum != null) {
				lastSwipe = idTagNum;
				response.setStatusCode(HttpStatus.SC_OK);
				EntityTemplate body = new EntityTemplate(new ContentProducer() {
					public void writeTo(final OutputStream outstream)
							throws IOException {
						OutputStreamWriter writer = new OutputStreamWriter(
								outstream, "UTF-8");
						writer.write("received swipe!");
						writer.flush();
					}
				});
				body.setContentType("text/html; charset=UTF-8");
				response.setEntity(body);
				logger.info("kiosk swiped idtag: " + idTagNum);
			} else {
				response.setStatusCode(HttpStatus.SC_OK);
				EntityTemplate body = new EntityTemplate(new ContentProducer() {

					public void writeTo(final OutputStream outstream)
							throws IOException {
						OutputStreamWriter writer = new OutputStreamWriter(
								outstream, "UTF-8");
						if (lastSwipe != null)
							writer.write(lastSwipe);
						else
							writer.write("");
						writer.flush();
					}

				});
				body.setContentType("text/html; charset=UTF-8");
				response.setEntity(body);

			}

		}

		if (method.equals("POST")) {
			logger.info("received post");

			logger.info("received post");
			List<NameValuePair> params = null;
			HashMap<String, String> paramsHash = new HashMap<String, String>();
			

			if (request instanceof HttpEntityEnclosingRequest) {
				HttpEntity entity = ((HttpEntityEnclosingRequest) request)
						.getEntity();
				params = URLEncodedUtils.parse(entity);
				for (NameValuePair param : params) {
					paramsHash.put(param.getName(), param.getValue());
				}
			}
			String idTagNum = paramsHash.get("tag");
			String name = paramsHash.get("name");
			if (idTagNum != null && name != null) {
				logger.info("new id tag from kiosk " + idTagNum + " " + name);
				// register
				try {
					dbAccess.addName(idTagNum, name);
				} catch (SQLException e) {
					logger.error("problem registering tag" + e.getMessage(), e);
				}

			} else {

				response.setStatusCode(HttpStatus.SC_OK);
				EntityTemplate body = new EntityTemplate(new ContentProducer() {

					public void writeTo(final OutputStream outstream)
							throws IOException {
						OutputStreamWriter writer = new OutputStreamWriter(
								outstream, "UTF-8");
						writer.write("<p>You did not enter either your name or ID number");
						writer.flush();
					}

				});
				body.setContentType("text/html; charset=UTF-8");
				response.setEntity(body);
	        	return;
			}

			response.setStatusCode(HttpStatus.SC_OK);
			EntityTemplate body = new EntityTemplate(new ContentProducer() {

				public void writeTo(final OutputStream outstream)
						throws IOException {
					OutputStreamWriter writer = new OutputStreamWriter(
							outstream, "UTF-8");
					writer.write("<p>Thank you for registering</p>");
					writer.flush();
				}

			});
			body.setContentType("text/html; charset=UTF-8");
			response.setEntity(body);

		}

	}
}
