/*
 *http://svn.apache.org/repos/asf/httpcomponents/httpcore/branches/4.0.x/httpcore/src/examples/org/apache/http/examples/ElementalHttpServer.java
 * ====================================================================
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package edu.columbia.cs.irt.rfidentify.server;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.URLDecoder;
import java.util.Date;
import java.util.Locale;

import org.apache.http.HttpEntity;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpException;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.MethodNotSupportedException;
import org.apache.http.entity.ContentProducer;
import org.apache.http.entity.EntityTemplate;
import org.apache.http.entity.FileEntity;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpRequestHandler;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author "Blake Arnold"
 *
 */
class HttpFileHandler implements HttpRequestHandler  {
    
    private final String docRoot;

	private static Logger logger = LoggerFactory.getLogger(HttpFileHandler.class);
    
    public HttpFileHandler(final String docRoot) {
        super();
        this.docRoot = docRoot;
    }
    
    public void handle(
            final HttpRequest request, 
            final HttpResponse response,
            final HttpContext context) throws HttpException, IOException {

        String method = request.getRequestLine().getMethod().toUpperCase(Locale.ENGLISH);
        if (!method.equals("GET") && !method.equals("HEAD") && !method.equals("POST")) {
            throw new MethodNotSupportedException(method + " method not supported"); 
        }
        String target = request.getRequestLine().getUri();

        if (request instanceof HttpEntityEnclosingRequest) {
            HttpEntity entity = ((HttpEntityEnclosingRequest) request).getEntity();
            byte[] entityContent = EntityUtils.toByteArray(entity);
            logger.info("Incoming entity content (bytes): " + entityContent.length);
        }

        File file = new File(this.docRoot, URLDecoder.decode(target, "UTF-8"));
        if (!file.exists()) {

            response.setStatusCode(HttpStatus.SC_NOT_FOUND);
            EntityTemplate body = new EntityTemplate(new ContentProducer() {
                
                public void writeTo(final OutputStream outstream) throws IOException {
                    OutputStreamWriter writer = new OutputStreamWriter(outstream, "UTF-8"); 
                    writer.write("<html><body><h1>");
                    writer.write("File not found");
                    writer.write("</h1></body></html>");
                    writer.flush();
                }
                
            });
            body.setContentType("text/html; charset=UTF-8");
            response.setEntity(body);
            logger.info("File " + file.getPath() + " not found");
            return;
            
        } else if (file.isDirectory()){ //List directory context
			File ind = new File(file, "index.html");
			if (ind.exists()) {
				file = ind;
			} else if(target.charAt(target.length()-1) == '/'){
				final String listing = listDirectory(file);
				response.setStatusCode(HttpStatus.SC_OK);
				EntityTemplate body = new EntityTemplate(new ContentProducer() {
	                
	                public void writeTo(final OutputStream outstream) throws IOException {
	                    OutputStreamWriter writer = new OutputStreamWriter(outstream, "UTF-8"); 
	                   	writer.write(listing);
	                   	writer.flush();
	                    
	                }
	                
	            });
	            body.setContentType("text/html; charset=UTF-8");
	            response.setEntity(body);
				logger.info("Listing directory" + file.getPath());
				return;
				
			} else { //add trailing slash to URI
	        	response.setStatusCode(HttpStatus.SC_MOVED_PERMANENTLY);
	        	response.setHeader("Location", target+"/");
	        	return;
			}
			
        	
        }  
        if (!file.canRead()) {
        
            
            response.setStatusCode(HttpStatus.SC_FORBIDDEN);
            EntityTemplate body = new EntityTemplate(new ContentProducer() {
                
                public void writeTo(final OutputStream outstream) throws IOException {
                    OutputStreamWriter writer = new OutputStreamWriter(outstream, "UTF-8"); 
                    writer.write("<html><body><h1>");
                    writer.write("Access denied");
                    writer.write("</h1></body></html>");
                    writer.flush();
                }
                
            });
            body.setContentType("text/html; charset=UTF-8");
            response.setEntity(body);
            logger.info("Cannot read file " + file.getPath());
            
        } else {
            
            response.setStatusCode(HttpStatus.SC_OK);
            FileEntity body = new FileEntity(file, "text/html");
            response.setEntity(body);
            logger.info("Serving file " + file.getPath());
            
        }
    }
    
    
	String listDirectory(File dir) {
		StringBuffer ps = new StringBuffer();
		ps.append("<TITLE>Directory listing</TITLE><P>\n");
		ps.append("<A HREF=\"..\">Parent Directory</A><BR>\n");
		String[] list = dir.list();
		for (int i = 0; list != null && i < list.length; i++) {
			File f = new File(dir, list[i]);
			if (f.isDirectory()) {
				ps.append("<A HREF=\"" + list[i] + "/\">" + list[i]
						+ "/</A><BR>");
			} else {
				ps.append("<A HREF=\"" + list[i] + "\">" + list[i]
								+ "</A><BR>");
			}
		}
		ps.append("<P><HR><BR><I>" + (new Date()) + "</I>");
		return ps.toString();
	}
    
}

