package edu.columbia.cs.irt.rfidentify.server;
import java.io.IOException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.apple.dnssd.*;

import edu.columbia.cs.irt.rfidentify.core.PropertyLoader;

/**
 * Register with zeroconfig service
 * Requires OS specific bindings
 * @author "Blake Arnold"
 *
 */
public class ZeroConfigRegister implements RegisterListener{

    private Logger logger = LoggerFactory.getLogger(this.getClass());
    private String name;
    private int port;
    private DNSSDRegistration dnss;
    private String service;
    
    /**
     * Initializes
     * @param port port number to register
     */
    public ZeroConfigRegister(int port){
    	this.port = port;
    	loadProperties();
    }
	@Override
	public void serviceRegistered(DNSSDRegistration registration, int flags,
			String serviceName, String regType, String domain) {
	    logger.info("Registered Name  : " + serviceName);
	    logger.info("           Type  : " + regType);
	    logger.info("           Domain: " + domain);
	    try {
			logger.info("           TXTRecord: " + registration.getTXTRecord());
		} catch (DNSSDException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	@Override
	public void operationFailed(DNSSDService service, int err) {
		logger.error("ZeroConfig registration failed " + err, service);
		
	}


	/**
	 *  starts registration
	 */
	public void start() {
		logger.info("Registration Starting");
		logger.info("Requested Name: " + name);
		logger.info("          Port: " + port);
		try {
			dnss = DNSSD.register(name, service, port, this);
		} catch (DNSSDException e) {
			logger.error("Could not register ZeroConfig service", e);
		}
	}
	
	/**
	 * unregisters
	 */
	public void stop(){
		logger.info("Registration Stopping");
		dnss.stop();
	}
	
	/**
	 * Loads registration properties
	 */
	private void loadProperties(){
		try {
			Properties p = PropertyLoader.loadProps("zeroconfig.properties");
			this.name = p.getProperty("name");
			this.service = p.getProperty("service");
			if(name==null){
				name = "rfidentify";
				service = "_rfid-srv._tcp";
			}
		} catch (IOException e) {
			logger.error("Could not load properties", e);
		}
	}

}
